/*
 * nasd_linux_general_simple.h
 *
 * Generic definitions useful to pretty much all NASD code
 *
 * Authors: Sean Levy, Marc Unangst
 */
/*
 * Copyright (c) of Carnegie Mellon University, 1999.
 *
 * Permission to reproduce, use, and prepare derivative works of
 * this software for internal use is granted provided the copyright
 * and "No Warranty" statements are included with all reproductions
 * and derivative works. This software may also be redistributed
 * without charge provided that the copyright and "No Warranty"
 * statements are included in all redistributions.
 *
 * NO WARRANTY. THIS SOFTWARE IS FURNISHED ON AN "AS IS" BASIS.
 * CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED AS TO THE MATTER INCLUDING, BUT NOT LIMITED
 * TO: WARRANTY OF FITNESS FOR PURPOSE OR MERCHANTABILITY, EXCLUSIVITY
 * OF RESULTS OR RESULTS OBTAINED FROM USE OF THIS SOFTWARE. CARNEGIE
 * MELLON UNIVERSITY DOES NOT MAKE ANY WARRANTY OF ANY KIND WITH RESPECT
 * TO FREEDOM FROM PATENT, TRADEMARK, OR COPYRIGHT INFRINGEMENT.
 */


#ifndef _NASD__NASD_LINUX_GENERAL_SIMPLE_H_
#define _NASD__NASD_LINUX_GENERAL_SIMPLE_H_

#if !defined(KERNEL) && NASD_NOASSERT == 0
#include <assert.h>
#endif /* !KERNEL && NASD_NOASSERT == 0 */

#ifndef KERNEL
#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <string.h>
#include <unistd.h>
/* #include <iovec.h> */
#include <sys/time.h>
#include <sys/types.h>
#endif /* !KERNEL */

#ifdef KERNEL
#include <linux/kernel.h>
#include <linux/string.h>
#include <linux/unistd.h>
#include <linux/limits.h>
#include <linux/uio.h>
#endif /* KERNEL */

#define bcopy(_src_,_dst_,_len_)  memcpy(_dst_,_src_,_len_)
#define bcmp(_buf1_,_buf2_,_len_) memcmp((void *)(_buf1_),(void *)(_buf2_),_len_)
#define bzero(_buf_,_len_)        memset((void *)(_buf_), 0, (_len_))

#ifndef KERNEL

#include <stdio.h>

#if NASD_RPC_PACKAGE == NASD_RPC_PACKAGE_DCE
/* abort() doesn't get along with the Linux DCE library.  this is a
   quick fix. */
#define abort()     exit(1)
#endif /* NASD_RPC_PACKAGE == NASD_RPC_PACKAGE_DCE */

#if NASD_NOASSERT == 0
#define NASD_ASSERT(_x_) { \
  if (!(_x_)) { \
    printf("Assertion check at file %s line %d\n", __FILE__, __LINE__); \
    printf("Assertion failed: %s\n", NASD_STRING(_x_)); \
    fflush(stdout); \
    abort(); \
    exit(1); \
  } \
}
#else /* NASD_NOASSERT == 0 */
#define NASD_ASSERT(x)   {/*noop*/}
#endif /* NASD_NOASSERT == 0 */
#define NASD_PANIC() { \
  printf("D'oh! Fatal error file %s line %d\n", __FILE__, __LINE__); \
  fflush(stdout); \
  fprintf(stderr, "D'oh! Fatal error file %s line %d\n", __FILE__, __LINE__); \
  fflush(stderr); \
  abort(); \
  exit(1); \
}

#endif /* !KERNEL */

#ifdef KERNEL

#ifdef NASD_ASSERT
#undef NASD_ASSERT
#endif /* NASD_ASSERT */


#define NASD_PANIC() { \
  panic("Fatal NASD error file %s line %d", __FILE__, __LINE__); \
}

#if NASD_NOASSERT == 0
#define NASD_ASSERT(_x_) { \
  if (!(_x_)) { \
    panic("NASD error at line %d file %s (failed asserting %s)", \
        __LINE__, __FILE__, #_x_); \
  } \
}
#else /* NASD_NOASSERT == 0 */
#define NASD_ASSERT(x) {/*noop*/}
#endif /* NASD_NOASSERT == 0 */

#endif /* KERNEL */

#define NASD_MAXPATHLEN PATH_MAX

#define NASD_UIO_MAXIOV UIO_MAXIOV

#endif /* !_NASD__NASD_LINUX_GENERAL_SIMPLE_H_ */

/* Local Variables:  */
/* indent-tabs-mode: nil */
/* tab-width: 2 */
/* End: */
